@@ -3,10 +3,10 @@ package Language::Farnsworth::Output;
 use strict;
 use warnings;
 
-use overload '""' => \&tostring;
+use overload '""' => \&tostring, "eq" => \&eq;
+
 
 use Data::Dumper;
-use Date::Manip;
 use Language::Farnsworth::Error;
 
 our %combos;
@@ -107,6 +107,16 @@ sub tostring
   return $self->getoutstring($value);
 }
 
+sub eq
+{
+  my $one = shift;
+  my $two = shift;
+  my $order = shift;
+
+  my $string = $one->tostring();
+  return $string eq $two;
+}
+
 #this takes a set of dimensions and returns what to display
 sub getoutstring
 {
